@@ -1,7 +1,7 @@
 #
 # Kiwi: a Framework and Enhanced Widgets for Python
 #
-# Copyright (C) 2001-2008 Async Open Source
+# Copyright (C) 2001-2012 Async Open Source
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -31,6 +31,7 @@ import gettext
 import locale
 import pickle
 
+import glib
 import gobject
 import pango
 import gtk
@@ -474,8 +475,9 @@ class Column(gobject.GObject):
                              (column, renderer_prop)):
         "To render the data of a cell renderer text"
         row = model[treeiter]
-        if column.editable_attribute:
-            data = column.get_attribute(row[COL_MODEL],
+        obj = row[COL_MODEL]
+        if column.editable_attribute and obj is not empty_marker:
+            data = column.get_attribute(obj,
                                         column.editable_attribute, None)
             if isinstance(renderer, gtk.CellRendererToggle):
                 renderer.set_property('activatable', data)
@@ -484,17 +486,20 @@ class Column(gobject.GObject):
             else:
                 raise AssertionError
 
-        obj = row[COL_MODEL]
         data = column.get_attribute(obj, column.attribute, None)
         text = column.as_string(data, obj)
 
-        if self._objectlist.cell_data_func:
+        if self._objectlist.cell_data_func and obj is not empty_marker:
             text = self._objectlist.cell_data_func(self, renderer, obj, text)
 
-        renderer.set_property(renderer_prop, text)
+        if obj is empty_marker:
+            if isinstance(renderer, gtk.CellRendererText):
+                renderer.set_property(renderer_prop, '')
+        else:
+            renderer.set_property(renderer_prop, text)
 
         if column.renderer_func:
-            column.renderer_func(renderer, row[COL_MODEL])
+            column.renderer_func(renderer, obj)
 
     def _cell_data_pixbuf_func(self, tree_column, renderer, model, treeiter,
                                (column, renderer_prop)):
@@ -641,8 +646,12 @@ class Column(gobject.GObject):
             issubclass(data_type, enum)):
             conv = converter.get_converter(data_type)
             text = conv.as_string(data, format=self.format or None)
-        else:
+        # Actually the expected data type depends on the renderer,
+        # but this is pragmatic workaround
+        elif data_type is bool:
             text = data
+        else:
+            text = str(data)
 
         return text
 
@@ -869,6 +878,7 @@ class _ContextMenu(gtk.Menu):
 COL_MODEL = 0
 
 _marker = object()
+empty_marker = object()
 
 
 class ObjectList(gtk.HBox):
@@ -1405,14 +1415,7 @@ class ObjectList(gtk.HBox):
     # Selection
     def _on_selection__changed(self, selection):
         "This method is used to proxy selection::changed to selection-changed"
-        mode = selection.get_mode()
-        if mode == gtk.SELECTION_MULTIPLE:
-            item = self.get_selected_rows()
-        elif mode in (gtk.SELECTION_SINGLE, gtk.SELECTION_BROWSE):
-            item = self.get_selected()
-        else:
-            raise AssertionError
-        self.emit('selection-changed', item)
+        self.update_selection()
 
     # ScrolledWindow
     def _on_scrolled_window__realize(self, widget):
@@ -1465,6 +1468,9 @@ class ObjectList(gtk.HBox):
 
     def _get_selection_or_selected_rows(self):
         selection = self._treeview.get_selection()
+        # This happens for unknown reasons in some crashreports
+        if selection is None:
+            return
         mode = selection.get_mode()
         if mode == gtk.SELECTION_MULTIPLE:
             item = self.get_selected_rows()
@@ -1579,10 +1585,23 @@ class ObjectList(gtk.HBox):
         "Return treemodel of the current list"
         return self._model
 
+    def set_model(self, model):
+        "Updates the model of the list and the treeview"
+        self._model = model
+        self._treeview.set_model(model)
+
     def get_treeview(self):
         "Return treeview of the current list"
         return self._treeview
 
+    def get_scrolled_window(self):
+        "Return the scrolled window containing the treeview"
+        return self._sw
+
+    def get_sortable(self):
+        "Returns if the ObjectList is going to be sortable"
+        return self._sortable
+
     def get_columns(self):
         return self._columns
 
@@ -1833,6 +1852,21 @@ class ObjectList(gtk.HBox):
             return [model[path][COL_MODEL] for (path,) in paths]
         return []
 
+    def update_selection(self):
+        mode = self._treeview.get_selection().get_mode()
+        if mode == gtk.SELECTION_MULTIPLE:
+            item = self.get_selected_rows()
+        elif mode in (gtk.SELECTION_SINGLE, gtk.SELECTION_BROWSE):
+            item = self.get_selected()
+        else:
+            raise AssertionError
+
+        # Skip emitting this
+        if (item is empty_marker or
+            type(item) == list and empty_marker in item):
+            return
+        self.emit('selection-changed', item)
+
     def add_list(self, instances, clear=True):
         """
         Allows a list to be loaded, by default clearing it first.
@@ -2283,7 +2317,8 @@ class ListLabel(gtk.HBox):
         value_format in the constructor.
         I also support the GMarkup syntax, so you can use "<b>%d</b>" if
         you want."""
-        self._value_widget.set_markup(self._value_format % value)
+        self._value_widget.set_markup(
+            self._value_format % glib.markup_escape_text(value))
 
     def get_value_widget(self):
         return self._value_widget
@@ -2301,9 +2336,6 @@ class ListLabel(gtk.HBox):
         # * size-allocate of treeview_columns header widget
         #
         tree_column = self._klist.get_treeview_column(self._column)
-        tree_column.connect('notify::width',
-                            self._on_treeview_column__notify_width)
-
         button = self._klist._get_column_button(tree_column)
         button.connect('size-allocate',
                        self._on_treeview_column_button__size_allocate)
@@ -2342,10 +2374,6 @@ class ListLabel(gtk.HBox):
     def _on_treeview_column_button__size_allocate(self, label, rect):
         self._resize(position=rect.x)
 
-    def _on_treeview_column__notify_width(self, treeview, pspec):
-        value = treeview.get_property(pspec.name)
-        self._resize(width=value)
-
     def _on_list__size_allocate(self, list, rect):
         self._resize(position=rect.x, width=rect.width)
 
