@@ -34,7 +34,7 @@ import gobject
 import gtk
 
 from kiwi.component import implements
-from kiwi.datatypes import currency
+from kiwi.currency import currency
 from kiwi.db.query import (NumberQueryState, StringQueryState,
                            DateQueryState, DateIntervalQueryState,
                            NumberIntervalQueryState, QueryExecuter)
@@ -42,6 +42,8 @@ from kiwi.enums import SearchFilterPosition
 from kiwi.interfaces import ISearchFilter
 from kiwi.python import enum
 from kiwi.ui.delegates import SlaveDelegate
+from kiwi.ui.lazyobjectlist import (LazyObjectListUpdater,
+                                    LazySummaryLabel)
 from kiwi.ui.objectlist import (ObjectList, ObjectTree,
                                 SummaryLabel, SearchColumn)
 from kiwi.ui.widgets.combo import ProxyComboBox
@@ -815,7 +817,8 @@ class StringSearchFilter(SearchFilter):
             entry.grab_focus()
             self.emit('changed')
         elif icon_pos == gtk.ENTRY_ICON_PRIMARY:
-            if not self._container:
+            # We don't need create popup filters if haven't search columns.
+            if not self._container or not hasattr(self._container, 'menu'):
                 return
             self._container.menu.popup(None, None,
                                        self._position_filter_menu, 0, event.time)
@@ -1076,10 +1079,12 @@ class SearchContainer(gtk.VBox):
             self.results_class = SearchResultsTree
 
         gtk.VBox.__init__(self)
-        self._columns = columns
-        self._search_filters = []
-        self._query_executer = None
         self._auto_search = True
+        self._columns = columns
+        self._lazy_updater = None
+        self._model = None
+        self._query_executer = None
+        self._search_filters = []
         self._summary_label = None
 
         search_filter = StringSearchFilter(_('Search:'), chars=chars,
@@ -1240,7 +1245,7 @@ class SearchContainer(gtk.VBox):
         """
         return self._primary_filter
 
-    def search(self):
+    def search(self, clear=True):
         """
         Starts a search.
         Fetches the states of all filters and send it to a query executer and
@@ -1250,10 +1255,23 @@ class SearchContainer(gtk.VBox):
             raise ValueError("A query executer needs to be set at this point")
         states = [(sf.get_state()) for sf in self._search_filters]
         results = self._query_executer.search(states)
-        self.add_results(results)
+        self.add_results(results, clear=clear)
         self.emit("search-completed", self.results, states)
         if self._summary_label:
-            self._summary_label.update_total()
+            if self._lazy_updater and len(self.results):
+                post = self.results.get_model().get_post_data()
+                if post is not None:
+                    self._summary_label.update_total(post.sum)
+            else:
+                self._summary_label.update_total()
+
+    def enable_lazy_search(self):
+        self._lazy_updater = LazyObjectListUpdater(
+            executer=self._query_executer,
+            search=self)
+        # Limits doesn't make sense when using lazy search, the idea
+        # is to always show everything.
+        self._query_executer.set_limit(-1)
 
     def set_auto_search(self, auto_search):
         """
@@ -1307,19 +1325,34 @@ class SearchContainer(gtk.VBox):
 
         if self._summary_label:
             self._summary_label.get_parent().remove(self._summary_label)
-        self._summary_label = SummaryLabel(klist=self.results,
-                                           column=column,
-                                           label=label,
-                                           value_format=format)
+        if self._lazy_updater:
+            summary_label_class = LazySummaryLabel
+        else:
+            summary_label_class = SummaryLabel
+        self._summary_label = summary_label_class(klist=self.results,
+                                                  column=column,
+                                                  label=label,
+                                                  value_format=format)
         parent.pack_end(self._summary_label, False, False)
         self._summary_label.show()
 
+    @property
+    def summary_label(self):
+        return self._summary_label
+
     def enable_advanced_search(self):
         self._create_advanced_search()
 
-    def add_results(self, results):
-        self.results.clear()
-        self.results.add_results(results)
+    def add_results(self, results, clear=True):
+        if clear:
+            if self._lazy_updater:
+                self._lazy_updater.restore_model()
+            self.results.clear()
+
+        if self._lazy_updater:
+            self._lazy_updater.add_results(results)
+        else:
+            self.results.add_results(results)
 
     def get_filter_states(self):
         dict_state = {}
