@@ -29,7 +29,8 @@ import datetime
 import gobject
 import pango
 
-from kiwi.datatypes import converter, number, ValueUnset, currency
+from kiwi.currency import currency
+from kiwi.datatypes import converter, number, ValueUnset
 from kiwi.decorators import deprecated
 from kiwi.enums import Alignment
 from kiwi.ui.proxywidget import ProxyWidgetMixin
@@ -246,6 +247,8 @@ class ProxyDateEntry(DateEntry, ValidatableProxyWidgetMixin):
     gsignal('validate', object, retval=object)
 
     def __init__(self):
+        self._in_do_changed = False
+
         DateEntry.__init__(self)
         ValidatableProxyWidgetMixin.__init__(self)
 
@@ -269,8 +272,12 @@ class ProxyDateEntry(DateEntry, ValidatableProxyWidgetMixin):
     gsignal('changed', 'override')
 
     def do_changed(self):
+        if self._in_do_changed:
+            return
+        self._in_do_changed = True
         self.chain()
         self.emit('content-changed')
+        self._in_do_changed = False
 
     # ProxyWidgetMixin implementation
 
